import {
  Body,
  Controller,
  Del,
  Get,
  Inject,
  Param,
  Post,
  Query,
} from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { AchieveService } from '../service/achieve.service';
import {
  CDTO_achieve,
  CVO_achieve,
  FVO_achieve,
  QDTO_achieve,
  QVO_achieve,
  UDTO_achieve,
  UVAO_achieve,
} from '../interface/achieve.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import get = require('lodash/get');
@ApiTags(['成果转化'])
@Controller('/achieve')
export class AchieveController extends BaseController {
  @Inject()
  service: AchieveService;

  @Post('/')
  @Validate()
  @ApiResponse({ type: CVO_achieve })
  async create(@Body() data: CDTO_achieve) {
    const dbData = await this.service.create(data);
    const result = new CVO_achieve(dbData);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_achieve })
  async query(
    @Query() filter: QDTO_achieve,
    @Query('skip') skip: number,
    @Query('limit') limit: number
  ) {
    // 依托单位只查选中自己的用户的数据
    const list = await this.service.query(filter, { skip, limit });
    const unit = get(filter, 'unit_id');
    let newList: object[] = [];
    if (unit) newList = list.filter(f => f.user?.unit === unit);
    else newList = list;
    const data = [];
    for (const i of newList) {
      const newData = new QVO_achieve(i);
      data.push(newData);
    }
    const total = newList.length;
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FVO_achieve })
  async fetch(@Param('id') id: string) {
    const data: any = await this.service.fetch(id);
    data.user = get(data.user, '_id');
    data.science = get(data.science, '_id');
    const result = new FVO_achieve(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UVAO_achieve })
  async update(@Param('id') id: string, @Body() body: UDTO_achieve) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
