import {
  Body,
  Controller,
  Del,
  Get,
  Inject,
  Param,
  Post,
  Query,
} from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { DictIndexService } from '../service/dictIndex.service';
import {
  CDTO_dictIndex,
  CVO_dictIndex,
  FVO_dictIndex,
  QDTO_dictIndex,
  QVO_dictIndex,
  UDTO_dictIndex,
  UVAO_dictIndex,
} from '../interface/dictIndex.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
@ApiTags(['字典目录'])
@Controller('/dictIndex')
export class DictIndexController extends BaseController {
  @Inject()
  service: DictIndexService;

  @Post('/')
  @Validate()
  @ApiResponse({ type: CVO_dictIndex })
  async create(@Body() data: CDTO_dictIndex) {
    const dbData = await this.service.create(data);
    const result = new CVO_dictIndex(dbData);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_dictIndex })
  async query(
    @Query() filter: QDTO_dictIndex,
    @Query('skip') skip: number,
    @Query('limit') limit: number
  ) {
    const list = await this.service.query(filter, { skip, limit });
    const data = [];
    for (const i of list) {
      const newData = new QVO_dictIndex(i);
      data.push(newData);
    }
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FVO_dictIndex })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FVO_dictIndex(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UVAO_dictIndex })
  async update(@Param('id') id: string, @Body() body: UDTO_dictIndex) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
