import {
  Body,
  Controller,
  Del,
  Get,
  Inject,
  Param,
  Post,
  Query,
} from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { GuideService } from '../service/guide.service';
import {
  CDTO_guide,
  CVO_guide,
  FVO_guide,
  QDTO_guide,
  QVO_guide,
  UDTO_guide,
  UVAO_guide,
} from '../interface/guide.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import get = require('lodash/get');
@ApiTags(['项目指南建议征集'])
@Controller('/guide')
export class GuideController extends BaseController {
  @Inject()
  service: GuideService;

  @Post('/')
  @Validate()
  @ApiResponse({ type: CVO_guide })
  async create(@Body() data: CDTO_guide) {
    const dbData = await this.service.create(data);
    const after = await this.service.afterCreate(dbData);
    const result = new CVO_guide(after);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_guide })
  async query(
    @Query() filter: QDTO_guide,
    @Query('skip') skip: number,
    @Query('limit') limit: number
  ) {
    const leach = await this.service.beforeQuery(filter);
    const list = await this.service.query(leach, {
      skip,
      limit,
      sort: { time: -1 },
    });
    const data = [];
    for (const i of list) {
      const newData = new QVO_guide(i);
      data.push(newData);
    }
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FVO_guide })
  async fetch(@Param('id') id: string) {
    const data: any = await this.service.fetch(id);
    data.user = get(data.user, '_id');
    const result = new FVO_guide(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UVAO_guide })
  async update(@Param('id') id: string, @Body() body: UDTO_guide) {
    const result = await this.service.updateOne(id, body);
    const after = await this.service.afterUpdate(result);
    return after;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
