import {
  Body,
  Controller,
  Del,
  Get,
  Inject,
  Param,
  Post,
  Query,
} from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { GuideRecordService } from '../service/guideRecord.service';
import {
  CDTO_guideRecord,
  CVO_guideRecord,
  FVO_guideRecord,
  QDTO_guideRecord,
  QVO_guideRecord,
  UDTO_guideRecord,
  UVAO_guideRecord,
} from '../interface/guideRecord.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
@ApiTags(['项目指南建议流程'])
@Controller('/guideRecord')
export class GuideRecordController extends BaseController {
  @Inject()
  service: GuideRecordService;

  @Post('/')
  @Validate()
  @ApiResponse({ type: CVO_guideRecord })
  async create(@Body() data: CDTO_guideRecord) {
    const before = await this.service.beforeCreate(data);
    const dbData = await this.service.create(before);
    const result = new CVO_guideRecord(dbData);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_guideRecord })
  async query(
    @Query() filter: QDTO_guideRecord,
    @Query('skip') skip: number,
    @Query('limit') limit: number
  ) {
    const list = await this.service.query(filter, { skip, limit });
    const data = [];
    for (const i of list) {
      const newData = new QVO_guideRecord(i);
      data.push(newData);
    }
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FVO_guideRecord })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FVO_guideRecord(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UVAO_guideRecord })
  async update(@Param('id') id: string, @Body() body: UDTO_guideRecord) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
