import {
  Body,
  Controller,
  Del,
  Get,
  Inject,
  Param,
  Post,
  Query,
} from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { PaperService } from '../service/paper.service';
import {
  CDTO_paper,
  CVO_paper,
  FVO_paper,
  QDTO_paper,
  QVO_paper,
  UDTO_paper,
  UVAO_paper,
} from '../interface/paper.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import get = require('lodash/get');
@ApiTags(['发表学术论文'])
@Controller('/paper')
export class PaperController extends BaseController {
  @Inject()
  service: PaperService;

  @Post('/')
  @Validate()
  @ApiResponse({ type: CVO_paper })
  async create(@Body() data: CDTO_paper) {
    const dbData = await this.service.create(data);
    const result = new CVO_paper(dbData);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_paper })
  async query(
    @Query() filter: QDTO_paper,
    @Query('skip') skip: number,
    @Query('limit') limit: number
  ) {
    // 依托单位只查选中自己的用户的数据
    const list = await this.service.query(filter, { skip, limit });
    const unit = get(filter, 'unit');
    let newList: object[] = [];
    if (unit) newList = list.filter(f => f.user?.unit === unit);
    else newList = list;
    const data = [];
    for (const i of newList) {
      const newData = new QVO_paper(i);
      data.push(newData);
    }
    const total = newList.length;
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FVO_paper })
  async fetch(@Param('id') id: string) {
    const data: any = await this.service.fetch(id);
    data.user = get(data.user, '_id');
    data.science = get(data.science, '_id');
    const result = new FVO_paper(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UVAO_paper })
  async update(@Param('id') id: string, @Body() body: UDTO_paper) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
