import {
  Body,
  Controller,
  Del,
  Get,
  Inject,
  Param,
  Post,
  Query,
} from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { ScienceService } from '../service/science.service';
import {
  CDTO_science,
  CVO_science,
  FVO_science,
  QDTO_science,
  QVO_science,
  UDTO_science,
  UVAO_science,
} from '../interface/science.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import get = require('lodash/get');
@ApiTags(['科研任务'])
@Controller('/science')
export class ScienceController extends BaseController {
  @Inject()
  service: ScienceService;

  @Post('/')
  @Validate()
  @ApiResponse({ type: CVO_science })
  async create(@Body() data: CDTO_science) {
    const dbData = await this.service.create(data);
    const result = new CVO_science(dbData);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_science })
  async query(
    @Query() filter: QDTO_science,
    @Query('skip') skip: number,
    @Query('limit') limit: number
  ) {
    const list = await this.service.query(filter, { skip, limit });
    const data = [];
    for (const i of list) {
      const newData = new QVO_science(i);
      data.push(newData);
    }
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FVO_science })
  async fetch(@Param('id') id: string) {
    const data: any = await this.service.fetch(id);
    data.user = get(data.user, '_id');
    data.science = get(data.science, '_id');
    const result = new FVO_science(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UVAO_science })
  async update(@Param('id') id: string, @Body() body: UDTO_science) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
