import { IMiddleware } from '@midwayjs/core';
import { Middleware, Inject } from '@midwayjs/decorator';
import { NextFunction, Context } from '@midwayjs/koa';
import { JwtService } from '@midwayjs/jwt';
import get = require('lodash/get');
import { FrameworkErrorEnum, ServiceError } from 'free-midway-component';
@Middleware()
export class addQeryMiddleware implements IMiddleware<Context, NextFunction> {
  @Inject()
  jwtService: JwtService;
  resolve() {
    return async (ctx: Context, next: NextFunction) => {
      const token = get(ctx, 'request.header.token');
      if (token) {
        const ui = await this.jwtService.decode(token);
        const rc = get(ui, 'role_type');
        const req = get(ctx, 'request.method');
        const url = get(ctx, 'request.path');
        if (req === 'GET' && rc === '3') {
          // 特殊处理这个接口因为里面有个字段也叫unit
          if (url !== '/lab_project/api/achieve') {
            ctx.query = { ...ctx.query, unit: get(ui, '_id') };
          } else ctx.query = { ...ctx.query, unit_id: get(ui, '_id') };
        } else if (req === 'GET' && rc === '2') {
          const user = get(ui, 'basic_id');
          if (!user)
            throw new ServiceError(
              '缺少项目基础信息数据',
              FrameworkErrorEnum.BAD_QUERY
            );
          ctx.query = { ...ctx.query, user };
        }
      }
      await next();
    };
  }

  static getName(): string {
    return 'addQery';
  }
}
