import { Provide, Inject } from '@midwayjs/decorator';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import get = require('lodash/get');
import {
  BaseService,
  FrameworkErrorEnum,
  ServiceError,
} from 'free-midway-component';
import { Basic } from '../entity/basic.entity';
import { User } from '../entity/user.entity';
import { JwtService } from '@midwayjs/jwt';
type modelType = ReturnModelType<typeof Basic>;
@Provide()
export class BasicService extends BaseService<modelType> {
  @Inject()
  jwtService: JwtService;
  @InjectEntityModel(Basic)
  model: modelType;

  @InjectEntityModel(User)
  userModel: ReturnModelType<typeof User>;
  /**
   * 与当前用户关联
   * @param body 前端数据
   * @param data 数据
   */
  async afterCreate(data) {
    const token = get(this.ctx, 'request.header.token');
    const user: any = await this.jwtService.decode(token);
    if (!user) {
      await this.delete(data._id);
      throw new ServiceError(
        '未找到用户信息',
        FrameworkErrorEnum.NOT_FOUND_DATA
      );
    }
    const user_id = get(user, '_id');
    if (!user_id)
      throw new ServiceError(
        '未找到当前用户的信息,请重新登陆',
        FrameworkErrorEnum.NOT_FOUND_DATA
      );
    const ud = await this.userModel.findOne({ user_id });
    if (ud) {
      await this.userModel.updateOne({ _id: ud._id }, { basic: data._id });
    } else {
      await this.delete(data._id);
      throw new ServiceError(
        '未找到当前用户信息',
        FrameworkErrorEnum.NOT_FOUND_DATA
      );
    }
  }

  async getSelfBasic() {
    const token = get(this.ctx, 'request.header.token');
    const user: any = await this.jwtService.decode(token);
    const basic_id = get(user, 'basic_id');
    if (!basic_id) return null;
    const data = await this.model.findById(basic_id);
    return data;
  }
}
