import { Provide, Inject } from '@midwayjs/decorator';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import { BaseService } from 'free-midway-component';
import { Guide } from '../entity/guide.entity';
import { GuideRecord } from '../entity/guideRecord.entity';
import { JwtService } from '@midwayjs/jwt';
import get = require('lodash/get');
import * as moment from 'moment';
type modelType = ReturnModelType<typeof Guide>;
@Provide()
export class GuideService extends BaseService<modelType> {
  @Inject()
  jwtService: JwtService;

  @InjectEntityModel(Guide)
  model: modelType;

  @InjectEntityModel(GuideRecord)
  guideRecordModel: ReturnModelType<typeof GuideRecord>;

  async beforeQuery(filter) {
    const token = get(this.ctx, 'request.header.token');
    const ui: any = await this.jwtService.decode(token);
    const code = get(ui, 'role_type');
    const ts = get(filter, 'status');
    let status = [];
    if (code === '3') {
      status = ['1', '-1', '2', '-2', '3'];
    } else if (code === '1') {
      status = ['2', '-2', '3'];
    } else if (code === '2') {
      const basic_id = get(ui, 'basic_id');
      filter.user = basic_id;
      return filter;
    } else if (code === '0') return filter;
    // 如果有要查询的指定状态,那就看这个状态在不在该角色的可视范围内.在的话可以查询,不在的话默认可视范围
    // 如果没有要查询的指定状态,那就直接默认可视范围
    if (ts) {
      if (!status.includes(ts)) filter.status = status;
    } else filter.status = status;
    return filter;
  }

  async afterCreate(data) {
    const token = get(this.ctx, 'request.header.token');
    const ui: any = await this.jwtService.decode(token);
    const obj = {
      guide: get(data, '_id'),
      status: '0',
      desc: '创建数据',
      time: moment().format('YYYY-MM-DD HH:mm:ss'),
      operator: ui._id,
    };
    await this.guideRecordModel.create(obj);
    return data;
  }

  async afterUpdate(data) {
    if (data.status === '0' || data.status === '1') {
      const token = get(this.ctx, 'request.header.token');
      const ui: any = await this.jwtService.decode(token);
      const obj = {
        guide: get(data, '_id'),
        status: data.status,
        desc: data.status === '0' ? '保存草稿' : '提交至依托管理员审核',
        time: moment().format('YYYY-MM-DD HH:mm:ss'),
        operator: ui._id,
      };
      await this.guideRecordModel.create(obj);
    }
    return data;
  }
}
